<?php

namespace Drupal\cloud\Commands;

use Drush\Commands\DrushCommands;
use Drupal\cloud\Service\CloudServiceInterface;
use Drupal\Component\Serialization\Yaml;

/**
 * A Drush Behat commands file.
 */
class BehatCommands extends DrushCommands {

  private const BEHAT_PATH = 'tests/src/Behat';

  /**
   * Cloud Service.
   *
   * @var \Drupal\cloud\Service\CloudServiceInterface
   */
  protected $cloudService;

  /**
   * BehatCommands constructor.
   *
   * @param \Drupal\cloud\Service\CloudServiceInterface $cloud_service
   *   Cloud Service.
   */
  public function __construct(CloudServiceInterface $cloud_service) {
    $this->cloudService = $cloud_service;
    parent::__construct();
  }

  /**
   * Preprocess behat test feature files. The path should be relative to /cloud.
   *
   * @param array $options
   *   An associative array of options whose values come from cli, config, etc.
   *
   * @option string $params_file
   *   The file defining parameters for templates.
   * @option string $templates_dir
   *   The directory where the templates of feature files exist.
   * @option string $output_dir
   *   The directory where this preprocessor creates files.
   *
   * @command cloud:behat:preprocess
   * @aliases cloud-behat-pre
   */
  public function preprocess(array $options = [
    'params_file' => self::BEHAT_PATH . '/behat_params.yml',
    'templates_dir' => self::BEHAT_PATH . '/features/templates',
    'output_dir' => self::BEHAT_PATH . '/features/tmp',
  ]): void {

    $base_path = \Drupal::service('extension.path.resolver')->getPath('module', 'cloud') . '/';

    $opt_params_file = $options['params_file'];
    $params_file = $base_path . $opt_params_file;

    $opt_templates_dir = $options['templates_dir'];
    $templates_dir = $base_path . $opt_templates_dir;

    $opt_output_dir = $options['output_dir'];
    $output_dir = $base_path . $opt_output_dir;

    if (!is_readable($params_file)) {
      $this->logger()->warning(dt('No such file exists or is readable for PARAMS_FILE: @opt_params_file', [
        '@opt_params_file' => $opt_params_file,
      ]));
      return;
    }

    if (!is_readable($templates_dir) || !is_dir($templates_dir)) {
      $this->logger()->warning(dt('No such directory exists or is readable for TEMPLATES_DIR: @opt_templates_dir', [
        '@opt_templates_dir' => $opt_templates_dir,
      ]));
      return;
    }

    $params = Yaml::decode(file_get_contents($params_file));
    if (empty($params)) {
      $this->logger()->warning(dt('No valid parameters are set in PARAMS_FILE: @opt_params_file', [
        '@opt_params_file' => $opt_params_file,
      ]));
      return;
    }

    if (!file_exists($output_dir)) {
      if (!mkdir($tmp_dir = $output_dir, 0755) && !is_dir($tmp_dir)) {
        $this->logger()->error(dt('Failed to create a directory of OUTPUT_DIR: @opt_output_dir'), [
          '@opt_output_dir' => $opt_output_dir,
        ]);
        return;
      }
      $this->logger()->notice(dt('Created: @opt', ['@opt' => $opt_output_dir]));
    }
    if (!is_dir($output_dir) || !is_writable($output_dir)) {
      $this->logger()->error(dt('Failed to find a writable directory of OUTPUT_DIR: @opt_output_dir', [
        '@opt_output_dir' => $opt_output_dir,
      ]));
      return;
    }

    if ($handle = opendir($templates_dir)) {
      $file_count = count(scandir($templates_dir)) - 2;
      $this->logger()->notice(dt('Processing @file_count file(s) under @opt_templates_dir...', [
        '@file_count' => $file_count,
        '@opt_templates_dir' => $opt_templates_dir,
      ]));
      $i = 0;
      while ($file = readdir($handle)) {
        if ($file !== '.' && $file !== '..') {
          $output = $output_dir . '/' . $file;
          $input = $templates_dir . '/' . $file;
          file_put_contents($output, $this->cloudService->renderTemplate(file_get_contents($input), $params));
          $i++;
          $this->logger()->notice(dt(' [@index/@file_count] Created: @opt_output_dir/@file', [
            '@file' => $file,
            '@file_count' => $file_count,
            '@index' => $i,
            '@opt_output_dir' => $opt_output_dir,
          ]));
        }
      }
      closedir($handle);
      $this->logger()->success(dt('Finished processing the templates.'));
    }
  }

}
